/**
 * @class Circle
 * @package window
 * @author Sebsatian Widelak
 */
var Circle = (function (window, undefined){
    var
        /**
         * CreateJS
         */
        Stage = createjs.Stage,
        Text = createjs.Text,
        Bitmap = createjs.Bitmap,
        Ticker = createjs.Ticker,
        /***
         * Box2D
         */
        b2Vec2 = Box2D.Common.Math.b2Vec2,
        b2BodyDef = Box2D.Dynamics.b2BodyDef,
        b2Body = Box2D.Dynamics.b2Body,
        b2FixtureDef = Box2D.Dynamics.b2FixtureDef,
        b2Fixture = Box2D.Dynamics.b2Fixture,
        b2World = Box2D.Dynamics.b2World,
        b2MassData = Box2D.Collision.Shapes.b2MassData,
        b2PolygonShape = Box2D.Collision.Shapes.b2PolygonShape,
        b2CircleShape = Box2D.Collision.Shapes.b2CircleShape,
        b2DebugDraw = Box2D.Dynamics.b2DebugDraw;

    /**
     *
     * @param bg
     * @param world
     * @param SCALE
     * @constructor
     */
    function Circle(bg, world, SCALE) {
        var me = this;
        me.SCALE = SCALE;
        me.world = world;
        me.view = new Bitmap(bg);
        me.view.regX = me.view.regY = 15;
        /**
         *  definicja fizyki (kolo)
         * @type {Box2D.Dynamics.b2FixtureDef}
         */
        me.fixDef = new b2FixtureDef;
            /**
             * @type {Box2D.Dynamics.b2BodyDef}
             */
            me.bodyDef = new b2BodyDef;

        /**
         * Gestosc (kg/m^2)
         * @type {number}
         */
        me.fixDef.density = 1;
        /**
         * wspolczynnik tarcia
         * @type {number}
         */
        me.fixDef.friction = .1;
        /**
         * elastycznosc ciala
         * @type {number}
         */
        me.fixDef.restitution = .5;
        /**
         * wlasciwosci ciala
         * @type {*}
         */
        me.bodyDef.type = b2Body.b2_dynamicBody;
        /**
         * Body definition
         * punktem poczatkowy dla b2D jest srodek
         * @type {number}
         */
        me.bodyDef.position.x = Math.random() * 800 / SCALE;
        me.bodyDef.position.y = 200 / SCALE;
        /**
         * Kolo
         * @type {*}
         */
        me.fixDef.shape = new b2CircleShape(15/me.SCALE);
        me.view.body = world.CreateBody(me.bodyDef);
        me.view.body.element_type = "Ball";
        me.view.body.CreateFixture(me.fixDef);
        me.view.onTick = function (e){
            tick.call(me, e, this);
        };
    }

    function tick(e, view){
        /**
         *
         * Eliminacja grawitacji na obiekcie
         */
        var body = this.view.body,
            fx = body.GetMass() * this.world.GetGravity().x,
            fy = body.GetMass() * this.world.GetGravity().y;
        body.ApplyForce(new b2Vec2(-fx, -fy), body.GetWorldCenter());
        /**
         * Ustawienie pozycji
         */
        view.x = this.view.body.GetPosition().x * this.SCALE;
        view.y = this.view.body.GetPosition().y * this.SCALE;
        /**
         * In radians
         * @type {number}
         */
        view.rotation = this.view.body.GetAngle() * (180/Math.PI);
    }
    return Circle;
}(window));