/**
 * @class Player
 * @package window
 * @author Sebsatian Widelak
 */
var Player = (function (window, undefined) {
    var
        /**
         * Easel
         */
            Stage = createjs.Stage,
        Text = createjs.Text,
        Bitmap = createjs.Bitmap,
        Ticker = createjs.Ticker,
        /***
         * Box2D
         */
            b2Vec2 = Box2D.Common.Math.b2Vec2,
        b2BodyDef = Box2D.Dynamics.b2BodyDef,
        b2Body = Box2D.Dynamics.b2Body,
        b2FixtureDef = Box2D.Dynamics.b2FixtureDef,
        b2Fixture = Box2D.Dynamics.b2Fixture,
        b2World = Box2D.Dynamics.b2World,
        b2MassData = Box2D.Collision.Shapes.b2MassData,
        b2PolygonShape = Box2D.Collision.Shapes.b2PolygonShape,
        b2CircleShape = Box2D.Collision.Shapes.b2CircleShape,
        b2DebugDraw = Box2D.Dynamics.b2DebugDraw,
        b2RevoluteJointDef = Box2D.Dynamics.Joints.b2RevoluteJointDef,
        b2DistanceJointDef = Box2D.Dynamics.Joints.b2DistanceJointDef,
        b2PrismaticJointDef = Box2D.Dynamics.Joints.b2PrismaticJointDef,
        /**
         * Const
         */
            DEGTORAD = 0.0174532925,
        FORCE = 1000;


    /**
     *
     * @param bg
     * @param world
     * @param SCALE
     * @constructor
     */
    function Player(bg, world, SCALE) {
        var me = this,
            /**
             *  definicja fizyki (cialo)
             * @type {Box2D.Dynamics.b2FixtureDef}
             */
                bodyFixDef = new b2FixtureDef,
            /**
             * definicja fizyki(uchwyt)
             */
                handlerFixDef = new b2FixtureDef,
            /**
             *  definicja fizyki (lewa noga)
             * @type {Box2D.Dynamics.b2FixtureDef}
             */
                leftLegFixDef = new b2FixtureDef,
            /**
             *  definicja fizyki (prawa noga)
             * @type {Box2D.Dynamics.b2FixtureDef}
             */
                rightLegFixDef = new b2FixtureDef,
            /**
             * @type {Box2D.Dynamics.b2BodyDef}
             */
                bodyBodyDef = new b2BodyDef,
            /**
             * @type {Box2D.Dynamics.b2BodyDef}
             */
                handlerBodyDef = new b2BodyDef,
            /**
             * @type {Box2D.Dynamics.b2BodyDef}
             */
                leftLegBodyDef = new b2BodyDef,
            /**
             * @type {Box2D.Dynamics.b2BodyDef}
             */
                rightLegBodyDef = new b2BodyDef,
            posX = Math.random() * 800 / SCALE,
            posY = 200 / SCALE,
            leftLegRevoluteJointDef = new b2RevoluteJointDef,
            handlerRevoluteJointDef = new b2RevoluteJointDef,
            leftHandlerLegRevoluteJointDef = new b2RevoluteJointDef,
            rightHandlerLegRevoluteJointDef = new b2RevoluteJointDef,
            rightLegRevoluteJointDef = new b2RevoluteJointDef;

        me.SCALE = SCALE;
        me.world = world;
        me.view = new Bitmap(bg);
        me.angle = 0;
        me.dir = {
            "Right": false,
            "Left": false,
            "Up": false,
            "Down": false
        };
        me.view.regX = 25;
        me.view.regY = 25;

        /**
         * Gestosc (kg/m^2)
         * @type {number}
         */
        bodyFixDef.density = 50;
        handlerFixDef.density = 0;
        leftLegFixDef.density = 50;
        rightLegFixDef.density = 50;
        /**
         * filter
         */
        handlerFixDef.filter.groupIndexity = -1;
        leftLegFixDef.filter.groupIndex = -1;
        rightLegFixDef.filter.groupIndexity = -1;
        /**
         * wspolczynnik tarcia
         * @type {number}
         */
        bodyFixDef.friction = 5;
        handlerFixDef.friction = 0;
        leftLegFixDef.friction = 5;
        rightLegFixDef.friction = 5;
        /**
         * elastycznosc ciala
         * @type {number}
         */
        bodyFixDef.restitution = .5;
        handlerFixDef.restitution = 0;
        leftLegFixDef.restitution = .5;
        rightLegFixDef.restitution = .5;
        /**
         * wlasciwosci ciala
         * @type {*}
         */
        bodyBodyDef.type = b2Body.b2_dynamicBody;
        handlerBodyDef.type = b2Body.b2_dynamicBody;
        leftLegBodyDef.type = b2Body.b2_dynamicBody;
        rightLegBodyDef.type = b2Body.b2_dynamicBody;


        bodyBodyDef.position.x = posX;
        bodyBodyDef.position.y = posY;
        handlerBodyDef.position.x = posX;
        handlerBodyDef.position.y = posY;
        leftLegBodyDef.position.x = posX + (20) / SCALE;
        leftLegBodyDef.position.y = posY - (15) / SCALE;
        rightLegBodyDef.position.x = posX + (20) / SCALE;
        rightLegBodyDef.position.y = posY + (15) / SCALE;

        /**
         * Kolo
         * @type {*}
         */
        bodyFixDef.shape = new b2CircleShape(25 / me.SCALE);
        handlerFixDef.shape = new b2PolygonShape();
        handlerFixDef.shape.SetAsBox(15 / SCALE, 20 / SCALE);
        leftLegFixDef.shape = new b2CircleShape(7 / me.SCALE);
        rightLegFixDef.shape = new b2CircleShape(7 / me.SCALE);
        /**
         * Body
         */
        me.view.body = world.CreateBody(bodyBodyDef);
        me.view.body.CreateFixture(bodyFixDef);
        /**
         * Handler
         */
        me.view.handler = world.CreateBody(handlerBodyDef);
        me.view.handler.CreateFixture(handlerFixDef);
        /**
         * Left leg
         */
        me.view.leftLeg = world.CreateBody(leftLegBodyDef);
        me.view.leftLeg.CreateFixture(leftLegFixDef);
        /**
         * Right leg
         */
        me.view.rightLeg = world.CreateBody(rightLegBodyDef);
        me.view.rightLeg.CreateFixture(rightLegFixDef);
        /**
         * Joints
         * Left
         */
        leftLegRevoluteJointDef.Initialize(
            me.view.body,
            me.view.leftLeg,
            me.view.body.GetWorldCenter()
        );
        leftLegRevoluteJointDef.enableMotor = false;
        me.leftLegRevoluteJoint = world.CreateJoint(leftLegRevoluteJointDef);
        /**
         * Right
         */
        rightLegRevoluteJointDef.Initialize(
            me.view.body,
            me.view.rightLeg,
            me.view.body.GetWorldCenter()
        );
        rightLegRevoluteJointDef.enableMotor = false;
        me.rightLegRevoluteJoint = world.CreateJoint(rightLegRevoluteJointDef);
        /**
         * left-handler
         */
        leftHandlerLegRevoluteJointDef.Initialize(
            me.view.leftLeg,
            me.view.handler,
            {
                x : me.view.handler.GetWorldCenter().x+ 25/SCALE,
                y : me.view.handler.GetWorldCenter().y
            }
        );
        leftHandlerLegRevoluteJointDef.enableMotor = false;
        me.leftHandlerLegRevoluteJoint = world.CreateJoint(leftHandlerLegRevoluteJointDef);
        /**
         * right-handler
         */
        rightHandlerLegRevoluteJointDef.Initialize(
            me.view.rightLeg,
            me.view.handler,
            {
                x : me.view.body.GetWorldCenter().x + 25/SCALE,
                y : me.view.body.GetWorldCenter().y
            }
        );
        rightHandlerLegRevoluteJointDef.enableMotor = false;
        me.rightHandlerLegRevoluteJoint = world.CreateJoint(rightHandlerLegRevoluteJointDef);
        /**
         * Handler
         */
        handlerRevoluteJointDef.Initialize(
            me.view.body,
            me.view.handler,
            me.view.body.GetWorldCenter()
        );
        handlerRevoluteJointDef.enableMotor = false;
        handlerRevoluteJointDef.collideConnected = false;
        me.handlerRevoluteJoint = world.CreateJoint(handlerRevoluteJointDef);
        console.log(me.handlerRevoluteJoint);
        /**
         * Joints - end
         */


        me.view.body.element_type = 'Player';
        me.view.onTick = function (e) {
            tick.call(me, e, this);
        };
    }

    /**
     * @private
     * @param e
     * @param view
     */
    function tick(e, view) {
        /**
         *
         * Eliminacja grawitacji na obiekcie
         */
        var me = this,
            body = me.view.body;
        /**
         * Przeliczanie kątów
         */
        me.rotate();
        /**
         * Run Forest run!
         */
        me.run();
        /**
         * Ustawienie pozycji canvasu
         */
        view.x = body.GetPosition().x * this.SCALE;
        view.y = body.GetPosition().y * this.SCALE
        /**
         * W radianach
         * @type {number}
         */
            //body.SetXForm(body.GetPosition(), this.angle);
        view.rotation = body.GetAngle() * (180 / Math.PI);
    }

    Player.prototype.setDir = function (dir, keydown) {
        var me = this;
        me.dir[dir] = keydown;
        switch (dir) {
            case 'Up':
            {
                me.dir['Down'] = false;
                break;
            }
            case 'Right':
            {
                me.dir['Left'] = false;
                break;
            }
            case 'Left':
            {
                me.dir['Right'] = false;
                break;
            }
            case 'Down' :
            {
                me.dir['Up'] = false;
                break;
            }
        }
    };

    Player.prototype.rotate = function (dir) {
        var me = this,
            body = this.view.body,
            handler = me.view.handler,
            nextAngle,
            totalRotation,
            desiredAngularVelocity,
            torque;

        if (me.dir['Right']) {
            me.angle = 0 * Math.PI;
        }
        if (me.dir['Down']) {
            me.angle = 0.5 * Math.PI;
        }
        if (me.dir['Left']) {
            me.angle = 1 * Math.PI;
        }
        if (me.dir['Up']) {
            me.angle = 1.5 * Math.PI;
        }

        if (me.dir['Right'] && me.dir['Up']) {
            me.angle = 1.75 * Math.PI;
        }
        if (me.dir['Right'] && me.dir['Down']) {
            me.angle = 0.25 * Math.PI;
        }
        if (me.dir['Down'] && me.dir['Left']) {
            me.angle = 0.75 * Math.PI;
        }
        if (me.dir['Left'] && me.dir['Up']) {
            me.angle = 1.25 * Math.PI;
        }
        nextAngle = body.GetAngle() + body.GetAngularVelocity() / 3;
        totalRotation = me.angle - nextAngle;//use angle in next time step
        while (totalRotation < -180 * DEGTORAD) totalRotation += 360 * DEGTORAD;
        while (totalRotation > 180 * DEGTORAD) totalRotation -= 360 * DEGTORAD;
        desiredAngularVelocity = totalRotation * 3;
        torque = body.GetInertia() * desiredAngularVelocity / (1 / 3.0);
        body.ApplyTorque(torque);
        handler.SetAngle(body.GetAngle());
    };

    Player.prototype.run = function () {
        var me = this,
            body = this.view.body,
            vel = body.GetLinearVelocity();

        if (me.dir['Right']) {
            body.ApplyForce(new b2Vec2(FORCE, 0), body.GetWorldCenter());
        }
        if (me.dir['Down']) {
            body.ApplyForce(new b2Vec2(0, FORCE), body.GetWorldCenter());
        }
        if (me.dir['Left']) {
            body.ApplyForce(new b2Vec2(-FORCE, 0), body.GetWorldCenter());
        }
        if (me.dir['Up']) {
            body.ApplyForce(new b2Vec2(0, -FORCE), body.GetWorldCenter());
        }
        if (me.dir['Right'] && me.dir['Up']) {
            body.ApplyForce(new b2Vec2(FORCE, -FORCE), body.GetWorldCenter());
        }
        if (me.dir['Right'] && me.dir['Down']) {
            body.ApplyForce(new b2Vec2(FORCE, FORCE), body.GetWorldCenter());
        }
        if (me.dir['Left'] && me.dir['Down']) {
            body.ApplyForce(new b2Vec2(-FORCE, FORCE), body.GetWorldCenter());
        }
        if (me.dir['Left'] && me.dir['Up']) {
            body.ApplyForce(new b2Vec2(-FORCE, -FORCE), body.GetWorldCenter());
        }

        //body.SetLinearVelocity(new b2Vec2(vel.x, vel.y - 2));
    }

    Player.prototype.halt = function () {
        var me = this,
            body = this.view.body,
            vel = body.GetLinearVelocity(),
            i1 = 1,
            i2 = 1;
        if (vel.x > 0) {
            i1 *= -1;
        }
        if (vel.y > 0) {
            i2 *= -1;
        }
        body.SetLinearVelocity(new b2Vec2(vel.x + i1, vel.y + i2));
    }

    return Player;
}(window));